"""
The official Geckoboard API client for Python

The Geckoboard API client can authenticate you
and connect you to our datasets API so that
you can create, update and delete datasets.
"""

from geckoboard.client import Client


def client(api_key, timeout=None):
    """
    Creates an instance of a Geckoboard API client

    The Geckoboard API client can authenticate you and connect you
    to our datasets API.

    Note
    ----
    You can find your API key by logging into the Geckoboard application
    and visiting the Account section.

    Parameters
    ----------
    api_key : str
        Your Geckoboard API key
    timeout : float
        Optional timeout parameter

    Returns
    -------
    Client
        A Geckoboard API client instance
    """
    return Client(api_key, timeout)



"""
Public class: DatasetsClient
"""

from geckoboard.dataset import Dataset


class DatasetsClient:
    """
    The datasets API client

    The datasets API client can find, create and delete your datasets.
    """

    def __init__(self, connection):
        self.__connection = connection

    def find_or_create(self, dataset_id, fields, unique_by=None):
        """
        Creates a new, or returns an existing dataset

        Searches for a dataset with a matching ID and schema. If a
        match is found that dataset is returned. Otherwise, a new
        dataset is created and returned.

        The dataset instance that is returned allows you
        to update or delete that dataset.

        Parameters
        ----------
        dataset_id : str
            The name of the dataset

        fields : dict
            The dataset schema (see https://developer.geckoboard.com/api-reference/python/#types)

        unique_by : list
            A list of schema keys (fields) that must have unique values for each entry

        Returns
        -------
        Dataset
            A new dataset instance

        Raises
        ------
        Exception
            If your API key is invalid or an existing dataset is found
            with a schema that doesn't match the one provided

        ConnectionError
            If you could not connect to the Geckoboard API
        """
        body = {
            'fields': fields,
        }

        if unique_by is not None:
            body['unique_by'] = unique_by

        self.__connection.put('/datasets/' + dataset_id, body)

        return Dataset(dataset_id, self.__connection)

    def delete(self, dataset_id):
        """
        Deletes a dataset by ID

        Allows you to delete a dataset with a given ID. You can
        also delete a dataset by calling the `delete` method
        on the dataset instance.

        Parameters
        ----------
        dataset_id : str
            The name of the dataset

        Returns
        -------
        bool
            True if the dataset was successfully deleted

        Raises
        ------
        Exception
            If your API key is invalid or no dataset was found
            that matches the ID provided

        ConnectionError
            If you could not connect to the Geckoboard API
        """
        self.__connection.delete('/datasets/' + dataset_id)

        return True



"""
Private: api access methods
"""

import requests

API_HOST = 'https://api.geckoboard.com'


def get(path, api_key, timeout=None):
    return requests.get(API_HOST + path, auth=(api_key, ''), timeout=timeout)


def delete(path, api_key, timeout=None):
    return requests.delete(API_HOST + path, auth=(api_key, ''), timeout=timeout)


def post(path, body, api_key, timeout=None):
    return requests.post(API_HOST + path, json=body, auth=(api_key, ''), timeout=timeout)


def put(path, body, api_key, timeout=None):
    return requests.put(API_HOST + path, json=body, auth=(api_key, ''), timeout=timeout)



"""
Public class: Client
"""

from geckoboard.connection import Connection
from geckoboard.datasets_client import DatasetsClient


class Client:
    """
    The Geckoboard API client

    The Geckoboard API client can authenticate you and connect you
    to our datasets API.

    Note
    ----
    You can find your API key by logging into the Geckoboard application
    and visiting the Account section.

    Parameters
    ----------
    api_key : str
        Your Geckoboard API key

    Attributes
    ----------
    datasets : DatasetsClient
        The datasets client can find, create and delete your datasets
    """

    def __init__(self, api_key, timeout=None):
        connection = Connection(api_key, timeout=timeout)

        self.__connection = connection
        self.datasets = DatasetsClient(connection)

    def ping(self):
        """
        Checks that your API key is valid and that you can reach the Geckoboard API

        Returns
        -------
        bool
            True if you are authenticated successfully

        Raises
        ------
        Exception
            If your API key is invalid

        ConnectionError
            If you could not connect to the Geckoboard API
        """
        self.__connection.get('/')

        return True



"""
Public class: Dataset
"""


class Dataset:
    """
    Update or delete an individual dataset.

    Parameters
    ----------
    dataset_id : str
        The name of the dataset
    """

    def __init__(self, dataset_id, connection):
        self.__id = dataset_id
        self.__connection = connection

    def put(self, items):
        """
        Replace all data entries in the Dataset

        Parameters
        ----------
        items : list
            A list of the entries to replace the current ones

        Returns
        -------
        bool
            True if the dataset was successfully updated

        Raises
        ------
        Exception
            If the items do not match the dataset schema

        ConnectionError
            If you could not connect to the Geckoboard API
        """
        body = {
            'data': items,
        }

        self.__connection.put('/datasets/' + self.__id + '/data', body)

        return True

    def post(self, items, delete_by=None):
        """
        Append new data entries to the Dataset

        Parameters
        ----------
        items : list
            A list of the entries to append to the current ones

        Returns
        -------
        bool
            True if the dataset was successfully updated

        Raises
        ------
        Exception
            If the items do not match the dataset schema

        ConnectionError
            If you could not connect to the Geckoboard API
        """
        body = {
            'data': items,
        }

        if delete_by is not None:
            body['delete_by'] = delete_by

        self.__connection.post('/datasets/' + self.__id + '/data', body)

        return True

    def delete(self):
        """
        Delete the dataset

        Returns
        -------
        bool
            True if the dataset was successfully updated

        Raises
        ------
        ConnectionError
            If you could not connect to the Geckoboard API
        """
        self.__connection.delete('/datasets/' + self.__id)

        return True



"""
Private class: Connection

A utility wrapper for containing API details and exposing
simplified API access methods.
"""

from geckoboard import api
from geckoboard import response_handler


class Connection:
    def __init__(self, api_key, timeout=None):
        self.api_key = api_key
        self.timeout = timeout

    def get(self, path):
        response = api.get(path, self.api_key, self.timeout)
        return self.handle_response(response)

    def delete(self, path):
        response = api.delete(path, self.api_key, self.timeout)
        return self.handle_response(response)

    def post(self, path, body):
        response = api.post(path, body, self.api_key, self.timeout)
        return self.handle_response(response)

    def put(self, path, body):
        response = api.put(path, body, self.api_key, self.timeout)
        return self.handle_response(response)

    @staticmethod
    def handle_response(response):
        if response.status_code < 200 or response.status_code >= 300:
            raise Exception(response_handler.get_error_message(response))

        return response



from setuptools import setup


def readme():
    with open('README.rst') as readme_file:
        return readme_file.read()


setup(
    name='geckoboard.py',
    version='1.1.0',
    description='Official Python client for the Geckoboard Datasets API',
    long_description=readme(),
    url='https://github.com/geckoboard/geckoboard-python',
    author='Geckoboard',
    author_email='dan@geckoboard.com',
    license='MIT',
    keywords=['geckoboard', 'datasets', 'api', 'python'],
    packages=['geckoboard'],
    install_requires=['requests'],
    tests_require=['nose', 'mock'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License'
    ]
)
